import serial.tools.list_ports
import mido
import serial
import time
from pynput.keyboard import Controller, Key

# Initialize the keyboard controller
keyboard = Controller()

def list_com_ports():
    """List all available COM ports."""
    com_ports = serial.tools.list_ports.comports()
    print("Available COM Ports:")
    for index, port in enumerate(com_ports):
        print(f"{index + 1}: Port: {port.device}, Description: {port.description}, HWID: {port.hwid}")
    return [port.device for port in com_ports]

def list_midi_ports():
    """List all available MIDI output ports."""
    midi_ports = mido.get_output_names()
    print("\nAvailable MIDI Output Ports:")
    for index, port in enumerate(midi_ports):
        print(f"{index + 1}: {port}")
    return midi_ports

def select_port(options, port_type):
    """Prompt the user to select a port from the given list."""
    while True:
        try:
            selection = int(input(f"\nSelect a {port_type} (1-{len(options)}): ")) - 1
            if 0 <= selection < len(options):
                return options[selection]
        except ValueError:
            pass
        print(f"Invalid selection. Please choose a number between 1 and {len(options)}.")

def process_midi_message(midi_message, description):
    """Process MIDI message and simulate key press if needed."""
    if description == "Simulate Option + Arrow Up":
        print("Simulating 'Option + Arrow Up' key press for tempo increase.")
        with keyboard.pressed(Key.alt):  # Simulate Option key
            keyboard.press(Key.up)
            keyboard.release(Key.up)
    elif description == "Simulate Option + Arrow Down":
        print("Simulating 'Option + Arrow Down' key press for tempo decrease.")
        with keyboard.pressed(Key.alt):  # Simulate Option key
            keyboard.press(Key.down)
            keyboard.release(Key.down)
    elif description == "Simulate 'K' Press":
        print("Simulating 'K' key press for metronome toggle.")
        keyboard.press('k')
        keyboard.release('k')
    elif description == "Simulate Space Bar":
        print("Simulating Space Bar key press.")
        keyboard.press(Key.space)
        keyboard.release(Key.space)
    else:
        print(f"Forwarding MIDI message: {description}")

def main():
    print("Scanning for available ports...")
    
    # List and select COM port
    com_ports = list_com_ports()
    if not com_ports:
        print("No COM ports found. Exiting.")
        return
    selected_com_port = select_port(com_ports, "COM Port")
    print(f"Selected COM Port: {selected_com_port}")
    
    # List and select MIDI output port
    midi_ports = list_midi_ports()
    if not midi_ports:
        print("No MIDI output ports found. Exiting.")
        return
    selected_midi_port = select_port(midi_ports, "MIDI Output Port")
    print(f"Selected MIDI Output Port: {selected_midi_port}")

    # Open the selected ports
    midi_output = mido.open_output(selected_midi_port)
    serial_port = serial.Serial(port=selected_com_port, baudrate=9600, timeout=0.1)

    try:
        print("\nListening for UART messages...")
        while True:
            if serial_port.in_waiting > 0:
                # Read the 3-byte MIDI message
                midi_message = serial_port.read(3)

                # Read description string
                description = ""
                while True:
                    char = serial_port.read(1).decode('utf-8')
                    if char == '\0':  # Null-terminated string
                        break
                    description += char

                # Process the MIDI message and description
                process_midi_message(midi_message, description)

                # Forward the MIDI message if it's not a key simulation
                if "Simulate" not in description:
                    midi_output.send(mido.Message.from_bytes(midi_message))
            time.sleep(0.01)  # Small delay to avoid high CPU usage
    except KeyboardInterrupt:
        print("\nExiting...")
    finally:
        serial_port.close()
        midi_output.close()

if __name__ == "__main__":
    main()


#---------------------------------------------------------------------------------------------------